#include "Actors.h"
#include "Game.h"
PlayerShotActor::PlayerShotActor( Math::Vector& pos , float speed , int power ) : BaseActor( pos , ACT_PlayerShot , 16 ) {
	mSpeed = speed;
	mPower = power;
}
PlayerShotActor::~PlayerShotActor(){
}
void PlayerShotActor::VOnUpdate( Game* game , int deltaMilliseconds ){
	mPos.y -= mSpeed;
	if( mPos.x < -20.0f || mPos.x > 470 || mPos.y < -30.0f || mPos.y > 620.0f ){
		mIsDeath = true;
	}
}
void PlayerShotActor::VOnDraw( Game* game , int deltaMilliseconds ){
	gShotPicts->mPlayerShot->draw(
		mPos.x - ( gShotPicts->mPlayerShot->width() / 2 ),
		mPos.y - ( gShotPicts->mPlayerShot->height() / 2 ) );
}
void PlayerShotActor::VListen( IEventData* eventData ){
	if( eventData->VGetType() == ET_Hit ){
		EVE_Hit* hitEvent = dynamic_cast< EVE_Hit* >( eventData );
		IActor* Obj1 = hitEvent->Obj1();
		IActor* Obj2 = hitEvent->Obj2();
		if( Obj1 == this ){
			if( Obj2->VGetType() == ACT_Enemy ){
				mIsDeath = true;
			}
		}
		if( Obj2 == this ){
			if( Obj1->VGetType() == ACT_Enemy ){
				mIsDeath = true;
			}
		}
	}
}

EnemyActor::EnemyActor( shared_ptr< EnemyData > data ) : BaseActor( getVector( 0 , 0 , 0 ) , ( int )ACT_Enemy , data->mHitSize ){
	mData = data;
	mEAPItr = mData->mEAPList.begin();
	mSMPItr = mData->mSMPList.begin();
	mCnt = 0;
	mPos.x = data->mX;
	mPos.y = data->mY;
	mHitCircle = data->mHitSize;
	mMaxHp = data->mLife;
}
EnemyActor::~EnemyActor(){}
void EnemyActor::VOnUpdate( Game* game , int deltaMilliseconds ){
	if( mData->mLife <= 0 ){
		if( mData->mIsBoss ){
			game->AddEffect( shared_ptr< IActor >( new DeathEffectActor( mPos , 6 , SOUND_Bom ) ) );
			game->EnemyShotConversionItem( ITEM_Point , 100 );
		}else{
			game->AddEffect( shared_ptr< IActor >( new DeathEffectActor( mPos , 3 , SOUND_Hit ) ) );
		}
		switch( mData->mItemType ){
			case ITEM_Point:
				game->AddItem( shared_ptr< IActor >( new PointItemActor( mPos , mData->mItemPoint ) ) );
				break;
			case ITEM_Power:
				game->AddItem( shared_ptr< IActor >( new PowerItemActor( mPos , mData->mItemPoint ) ) );
				break;
			case ITEM_Life:
				game->AddItem( shared_ptr< IActor >( new LifeItemActor( mPos , mData->mItemPoint ) ) );
				break;
		}
		mIsDeath = true;
	}else{
		( *mEAPItr )->VOnUpdate( this );
		if( ( *mEAPItr )->VIsEnd() ){
			++mEAPItr;
		}
		if( ( *mSMPItr )->VGetType() == SMPT_Loop ){
			mSMPItr = mData->mSMPList.begin();
			while( mSMPItr != mData->mSMPList.end() ){
				( *mSMPItr )->VInit();
				++mSMPItr;
			}
			mSMPItr = mData->mSMPList.begin();
		}
		( *mSMPItr )->OnUpdate( game , this );
		if( ( *mSMPItr )->VIsEnd() ){
			++mSMPItr;
		}
	}
	if( mPos.x < -20.0f || mPos.x > 470.0f || mPos.y < -30.0f || mPos.y > 620.0f ){
		mIsDeath = true;
	}
	++mCnt;
}
void EnemyActor::VOnDraw( Game* game , int deltaMilliseconds ){
	mData->mImage->draw( mPos.x - ( mData->mImage->width() / 2 ),
		mPos.y - ( mData->mImage->height() / 2 ) );
}
void EnemyActor::VListen( IEventData* eventData ){
	if( eventData->VGetType() == ET_Hit ){
		EVE_Hit* hitEvent = dynamic_cast< EVE_Hit* >( eventData );
		IActor* Obj1 = hitEvent->Obj1();
		IActor* Obj2 = hitEvent->Obj2();
		if( Obj1 == this ){
			if( Obj2->VGetType() == ACT_Player ){
				PlayerActor* player = dynamic_cast< PlayerActor* >( Obj2 );
				if( player->GetMuteki() == 0 ){
					mData->mLife -= player->GetPower();
				}
			}
			if( Obj2->VGetType() == ACT_PlayerShot ){
				PlayerShotActor* pShot = dynamic_cast< PlayerShotActor* >( Obj2 );
				mData->mLife -= pShot->GetPower();
			}
		}else if( Obj2 == this ){
			if( Obj1->VGetType() == ACT_Player ){
				PlayerActor* player = dynamic_cast< PlayerActor* >( Obj1 );
				if( player->GetMuteki() == 0 ){
					mData->mLife -= player->GetPower();
				}
			}
			if( Obj1->VGetType() == ACT_PlayerShot ){
				PlayerShotActor* pShot = dynamic_cast< PlayerShotActor* >( Obj1 );
				mData->mLife -= pShot->GetPower();
			}
		}
	}
}
PlayerActor::PlayerActor( Game* game ) : BaseActor( getVector( 225.0f , 480.0f , 0 ) , ( int ) ACT_Player , 4 ){
	mPad = Manager::instance()->getPad( 0 );
	mKeyboard = Manager::instance()->getKeyboard();
	mImage = shared_ptr< Image >( new Image( "Pict/Game/Jiki.png" ) );
	mShot = shared_ptr< Image >( new Image( "Pict/Game/PlayerShot.png" ) ); 
	mMuteki = 60;
	mHitCircle = 4;
	mShotCnt = 0;
	mGame = game;
	mScore = 0;
	mPower = 1 * 1000;
	mPointGetSound = shared_ptr< Sound >( new Sound( gSoundDatas->mPointGet->handle() ) );
	mPowerGetSound = shared_ptr< Sound >( new Sound( gSoundDatas->mPowerGet->handle() ) );
	mDeathSound = shared_ptr< Sound >( new Sound( gSoundDatas->mBom->handle() ) );
	mPointGetSound->setVolume( gSoundDatas->mVolume );
	mPowerGetSound->setVolume( gSoundDatas->mVolume );
	mDeathSound->setVolume( gSoundDatas->mVolume );
}
PlayerActor::~PlayerActor(){}
int PlayerActor::GetMuteki() const{ return mMuteki; }
void PlayerActor::VOnUpdate( Game* game , int deltaMilliseconds ){
	float x = 0.0f , y = 0.0f;
	mShift = false;
	if( mPad->isOn( Pad::BUT_UP ) ||
		mKeyboard->isOn( Keyboard::KEY_UP ) )
	{
		y -= 1.0f;
	}
	if( mPad->isOn( Pad::BUT_DOWN ) ||
		mKeyboard->isOn( Keyboard::KEY_DOWN ) )
	{
		y += 1.0f;
	}
	if( mPad->isOn( Pad::BUT_LEFT ) ||
		mKeyboard->isOn( Keyboard::KEY_LEFT ) )
	{
		x -= 1.0f;
	}
	if( mPad->isOn( Pad::BUT_RIGHT ) ||
		mKeyboard->isOn( Keyboard::KEY_RIGHT ) )
	{
		x += 1.0f;
	}
	if( mPad->isOn( Pad::BUT_LB ) ||
		mKeyboard->isOn( Keyboard::KEY_LSHIFT ) ||
		mKeyboard->isOn( Keyboard::KEY_RSHIFT ) )
	{
		mShift = true;
	}
	if( x != 0.0f || y != 0.0f ){
		float speed = 4.0f;
		if( mShift ){ speed /= 2; }
		float theta = atan2f( y , x );
		mPos.y += sinf( theta ) * speed;
		mPos.x += cosf( theta ) * speed;
	}
	if( mPos.x < 0 ) mPos.x = 0.0f;
	if( mPos.x > 450.0f ) mPos.x = 450.0f;
	if( mPos.y < 0 ) mPos.y = 0.0f;
	if( mPos.y > 560.0f ) mPos.y = 560.0f;
	if( ( mPad->isOn( Pad::BUT_A ) ||
		mKeyboard->isOn( Keyboard::KEY_Z ) ) &&
		mMuteki == 0 )
	{
		if( mShotCnt == 0 ){
			shared_ptr< PlayerShotActor > shot( new PlayerShotActor( Math::getVector( mPos.x , mPos.y - 10 , mPos.z ) , 12 , mPower ) );
			game->AddPlayerShot( shot );
			mShotCnt = 3;
		}
	}
	if( mShotCnt > 0 ){
		--mShotCnt;
	}
	if( mMuteki > 0 ){
		--mMuteki;
	}
}
void PlayerActor::VOnDraw( Game* game , int deltaMilliseconds ){
	mImage->draw(( int )( mPos.x ) - ( mImage->width() / 2 ),
		( int )( mPos.y ) - ( mImage->height() / 2 ) );
	if( mShift ){
		DrawCircle( ( int )( mPos.x ) , ( int )( mPos.y ) , mHitCircle , GetColor( 255 , 0 , 0 ) );
	}
}
void PlayerActor::VListen( IEventData* eventData ){
	if( eventData->VGetType() == ET_Hit ){
		if( mMuteki == 0 ){
			EVE_Hit* hitEvent = dynamic_cast< EVE_Hit* >( eventData );
			IActor* Obj1 = hitEvent->Obj1();
			IActor* Obj2 = hitEvent->Obj2();
			if( Obj1 == this ){
				if( Obj2->VGetType() == ACT_Enemy ){
					--mLife;
					mMuteki = 60;
					mGame->ClearEnemyShot();
					mDeathSound->play( Sound::BACK );
				}
				if( Obj2->VGetType() == ACT_EnemyShot ){
					--mLife;
					mMuteki = 60;
					mGame->ClearEnemyShot();
					mDeathSound->play( Sound::BACK );
				}
			}
			if( Obj2 == this ){
				if( Obj1->VGetType() == ACT_Enemy ){
					--mLife;
					mMuteki = 60;
					mGame->ClearEnemyShot();
					mDeathSound->play( Sound::BACK );
				}
				if( Obj1->VGetType() == ACT_EnemyShot ){
					--mLife;
					mMuteki = 60;
					mGame->ClearEnemyShot();
					mDeathSound->play( Sound::BACK );
				}
			}
		}
		return;
	}
}
void ErekaLineShot::VOnUpdate( Game* game , int deltaMilliseconds ){
	if( mCnt == mDelay ){
		Math::Vector playerPos = game->GetPlayer()->VGetPos();
		float x = playerPos.x - mPos.x;
		float y = playerPos.y - mPos.y;
		mAngle = atan2f( y , x );
		mSpeed = m_Speed;
	}
	mPos.x += cosf( mAngle ) * mSpeed;
	mPos.y += sinf( mAngle ) * mSpeed;
	DeathCheck();
	++mCnt;
}
void ErekaLineShot::VOnDraw( Game* game , int deltaMilliseconds ){
	gShotPicts->mStarPinkShot->draw( mPos.x - gShotPicts->mNormalShot->width() / 2,
		mPos.y - gShotPicts->mNormalShot->height() / 2 );
}
void BaseItemActor::VOnUpdate( Game* game , int deltaMilliseconds ){
	if( mGuided ){
		shared_ptr< PlayerActor > Player = game->GetPlayer();
		float x = Player->VGetPos().x - mPos.x;
		float y = Player->VGetPos().y - mPos.y;
		float angle = atan2f( y , x );
		mPos.x += cosf( angle ) * mSpeed;
		mPos.y += sinf( angle ) * mSpeed;
	}else{
		mPos.y += 2.0f;
	}
	if( mPos.x < -20.0f ||
		mPos.x > 470.0f ||
		mPos.y < -30.0f ||
		mPos.y > 620.0f )
	{
		mIsDeath = true;
	}
}