#pragma once
#include "EnemyActionPattern.h"
#include "ShotMovePattern.h"
#include "Std.h"
enum ActorType{
	ACT_Enemy,
	ACT_EnemyShot,
	ACT_Player,
	ACT_PlayerShot,
	ACT_PlayerBlade,
	ACT_Effect,
	ACT_Item,
	ACT_None
};
enum ItemType{
	ITEM_Point,
	ITEM_Power,
	ITEM_Life,
	ITEM_None
};
class Game;
typedef unsigned int ActorId;
class IActor{
public:
	virtual void VSetPos( const Math::Vector& pos ) = 0;
	virtual void VSetID( ActorId id ) = 0;
	
	virtual ~IActor(){};
	virtual Math::Vector const & VGetPos() const = 0;
	virtual const int VGetType() = 0;
	virtual ActorId VGetID() = 0;
	virtual void VOnUpdate( Game* game , int deltaMilliseconds ) = 0;
	virtual void VOnDraw( Game* game , int deltaMilliseconds ) =0;
	virtual bool VIsDeath() = 0;
	virtual int VGetHitCircle() const = 0;
};
class BaseActor : public IActor{
protected:
	Math::Vector mPos;
	ActorId mActorId;
	int mType;
	bool mIsDeath;
	int mHitCircle;

	virtual void VSetID( ActorId id ){ mActorId = id; }
public:
	BaseActor( Math::Vector& pos , int type , int hitCircle ) : mPos( pos ) , mType( type ) , mIsDeath( false ) , mHitCircle( hitCircle ){};
	~BaseActor(){}
	virtual void VSetPos( const Math::Vector& pos ){ mPos = pos; }
	virtual Math::Vector const & VGetPos() const{ return mPos; }
	virtual const int VGetType(){ return mType; }
	virtual ActorId VGetID(){ return mActorId; }
	virtual void VOnUpdate( Game* game , int deltaMilliseconds ){}
	virtual bool VIsDeath(){ return mIsDeath; }
	virtual int VGetHitCircle() const{ return mHitCircle; }
};
struct EnemyData;
struct StoryLine{
	std::string mName;
	int mImageNo;
	std::string mText;
};
struct StoryChara{
	shared_ptr< Image > mImage[ 5 ];
	bool mIsLeft;
	std::string mName;
};
struct StoryData{
	std::map< std::string , StoryChara > mCharaMap;
	std::vector< StoryLine > mLines;
	int mBossNo;
	StoryData(){
	}
	StoryData( std::string filename , int bossNo ) : mBossNo( bossNo ){
		CSVLoader csvl( filename.c_str() );
		int allLine = csvl.allLine();
		for( int i = 0 ; i < allLine ; ++i ){
			if( csvl.intData( 0 , i ) == 0 ){
				//wb_[
				std::string charaName = csvl.strData( 1 , i );
				bool left = ( csvl.strData( 2 , i ) ) == "" ? true : false;
				std::string charaImages[ 5 ];
				for( int j = 0 ; j < 5 ; ++j ){
					charaImages[ j ] = csvl.strData( j + 3 , i );
				}
				StoryChara chara;
				chara.mName = charaName;
				chara.mIsLeft = left;
				for( int j = 0 ; j < 5 ; ++j ){
					std::string imgPath = "Pict/Game/";
					imgPath += charaImages[ j ];
					chara.mImage[ j ] = shared_ptr< Image >( new Image( imgPath.c_str() ) );
				}
				mCharaMap.insert( std::pair< std::string , StoryChara >( charaName , chara ) );
			}else if( csvl.intData( 0 , i ) == 1 ){
				StoryLine line;
				line.mName = csvl.strData( 1 , i );
				line.mImageNo = csvl.intData( 2 , i );
				line.mText = csvl.strData( 3 , i );
				mLines.push_back( line );
			}
		}
	}
	~StoryData(){
	}
};
struct MapData{
	shared_ptr< Image > mBg;
	int mWidth;
	int mHeight;
	int mOffsetX;
	int mOffesetY;
	int mStageNo;
	std::string mNextStageName;
	std::vector< shared_ptr< EnemyData > > mEnemyDatas;
	shared_ptr< Sound > mRoadSound;
	shared_ptr< Sound > mBossSound;
	StoryData mStory;
	int mBossNo;
};
struct EnemyData{
	shared_ptr< Image > mImage;//摜
	int mNo;
	int mHitSize;//蔻TCY
	int mLife;//Ct|Cg
	int mTime;//oꎞ
	int mNotDeathTime;//G
	int mX;//oꏊ
	int mY;//oꏊ
	bool mIsBoss;
	ItemType mItemType;
	int mItemPoint;
	std::vector< shared_ptr< EnemyActionPattern > > mEAPList;
	std::vector< shared_ptr< IShotMovePattern > > mSMPList;
};
class MapDataFactory{
public:
	MapDataFactory( const char* datasName ) : mMapData( new CSVLoader( datasName ) ){};
	~MapDataFactory(){}
	void setEnemyData( const char* filename ){
		CSVLoader list( filename );
		std::string Path_Head = "Data/Game/";
		for( int i = 0 ; i < list.allLine() ; ++i ){
			if( list.intData( 0 , i ) == mEnemyDatas.size() ){
				std::string Path = Path_Head + list.strData( 1 , i );
				mEnemyDatas.push_back( shared_ptr< CSVLoader >( new CSVLoader( Path.c_str() ) ) );
			}
		}
	}
	MapData GetMapData(){
		MapData ret;
		ret.mStageNo = mMapData->intData( 0 , 0 );
		std::string bgPath = "Pict/Game/";
		bgPath += mMapData->strData( 1 , 0 );
		ret.mBg = shared_ptr< Image >( new Image( bgPath.c_str() ) );
		ret.mWidth = mMapData->intData( 2 , 0 );
		ret.mHeight = mMapData->intData( 3 , 0 );
		ret.mOffsetX = mMapData->intData( 4 , 0 );
		ret.mOffesetY = mMapData->intData( 5 , 0 );
		ret.mNextStageName = mMapData->strData( 6 , 0 );
		std::string SoundPath = "Music/Game/";
		SoundPath += mMapData->strData( 7 , 0 );
		ret.mRoadSound = shared_ptr< Sound >( new Sound( SoundPath.c_str() ) );
		SoundPath = "Music/Game/";
		SoundPath += mMapData->strData( 8 , 0 );
		ret.mBossSound = shared_ptr< Sound >( new Sound( SoundPath.c_str() ) );
		ret.mBossNo = mMapData->intData( 9 , 0 );
		std::string StoryPath = "Data/Game/";
		StoryPath += mMapData->strData( 10 , 0 );
		ret.mStory = StoryData( StoryPath.c_str() , ret.mBossNo );
		const int end = mMapData->allLine();
		for( unsigned int i = 1 ; i < end ; ++i ){
			ret.mEnemyDatas.push_back( CreateEnemyData( mMapData->intData( 0 , i ) ) );
		}
		return ret;
	}
	shared_ptr< EnemyData > GetEnemyData( int EnemyId ){
		shared_ptr< EnemyData > ret( new EnemyData() );
		const int endEnemy = mEnemyDatas[ EnemyId ]->allLine();
		for( int i = 0 ; i < endEnemy ; ++i ){
			if( mEnemyDatas[ EnemyId ]->intData( 0 , i ) == 0 ){
				std::string imageStr = "Pict/Game/";
				imageStr.append( mEnemyDatas[ EnemyId ]->strData( 1 , i ) );
				std::map< std::string , shared_ptr< Image > >::iterator imageItr = mImages.find( imageStr );
				shared_ptr< Image > image;
				if( imageItr == mImages.end() ){
					image = shared_ptr< Image >( new Image( imageStr.c_str() ) );
					mImages.insert( std::pair< std::string , shared_ptr< Image > >( imageStr , image ) );
				}else{
					image = ( *imageItr ).second;
				}
				ret->mImage = shared_ptr< Image >( new Image( imageStr.c_str() ) );
				ret->mHitSize = mEnemyDatas[ EnemyId ]->intData( 2 , i );
				ret->mLife = mEnemyDatas[ EnemyId ]->intData( 3 , i ) * 1000;
				ret->mIsBoss = mEnemyDatas[ EnemyId ]->boolData( 4 , i );
				std::string itemTypeStr = mEnemyDatas[ EnemyId ]->strData( 5 , i );
				ItemType itemType;
				if( itemTypeStr == "Point" ){ itemType = ITEM_Point; }
				else if( itemTypeStr == "Power" ){ itemType = ITEM_Power; }
				else if( itemTypeStr == "Life" ){ itemType = ITEM_Life; }
				else{ itemType = ITEM_None; }
				ret->mItemType = itemType;
				ret->mItemPoint = mEnemyDatas[ EnemyId ]->intData( 6 , i );
			}

			if( mEnemyDatas[ EnemyId ]->intData( 0 , i  ) == 1 ){
				std::string eapName = mEnemyDatas[ EnemyId ]->strData( 1 , i );
				if( eapName == "Move" ){
					int time = mEnemyDatas[ EnemyId ]->intData( 2 , i );
					int speed = mEnemyDatas[ EnemyId ]->intData( 3 , i );
					int angle = mEnemyDatas[ EnemyId ]->intData( 4 , i );
					bool endless = ( time == 0 ) ? true : false;
					ret->mEAPList.push_back( shared_ptr< EnemyActionPattern >( new EA_Move( time , speed , angle , endless ) ) );
				}else if( eapName == "Standby" ){
					int time = mEnemyDatas[ EnemyId ]->intData( 2 , i );
					ret->mEAPList.push_back( shared_ptr< EnemyActionPattern >( new EA_Standby( time ) ) );
				}else if( eapName == "Guided" ){
					int time = mEnemyDatas[ EnemyId ]->intData( 2 , i );
					int speed = mEnemyDatas[ EnemyId ]->intData( 3 , i );
					bool endless = ( time == 0 ) ? true : false;
					ret->mEAPList.push_back( shared_ptr< EnemyActionPattern >( new EA_Guided( time , speed , endless ) ) );
				}else if( eapName == "Kill" ){
					ret->mEAPList.push_back( shared_ptr< EnemyActionPattern >( new EA_Kill() ) );
				}
			}
			if( mEnemyDatas[ EnemyId ]->intData( 0 , i ) == 2 ){
				std::string smpData = mEnemyDatas[ EnemyId ]->strData( 1 , i );
				if( smpData == "Standby" ){
					int time = mEnemyDatas[ EnemyId ]->intData( 2 , i );
					bool endless = ( time == 0 ) ? true : false;
					ret->mSMPList.push_back( shared_ptr< IShotMovePattern >( new SM_Standby( time , endless ) ) );
				}else if( smpData == "StraightToP" ){
					int time = mEnemyDatas[ EnemyId ]->intData( 2 , i );
					int speed = mEnemyDatas[ EnemyId ]->intData( 3 , i );
					int freq = mEnemyDatas[ EnemyId ]->intData( 4 , i );
					bool endless = ( time == 0 ) ? true : false;
					ret->mSMPList.push_back( shared_ptr< IShotMovePattern >( new SM_StraightToP( time , speed , freq , endless ) ) );
				}else if( smpData == "StraightToFirstPos" ){
					int time = mEnemyDatas[ EnemyId ]->intData( 2 , i );
					int speed = mEnemyDatas[ EnemyId ]->intData( 3 , i );
					int freq = mEnemyDatas[ EnemyId ]->intData( 4 , i );
					bool endless = ( time == 0 ) ? true : false;
					ret->mSMPList.push_back( shared_ptr< IShotMovePattern >( new SM_StraightToFirstPos( time , speed , freq ) ) );
				}else if( smpData == "StrAcceToP" ){
					int time = mEnemyDatas[ EnemyId ]->intData( 2 , i );
					int speed = mEnemyDatas[ EnemyId ]->intData( 3 , i );
					int acce = mEnemyDatas[ EnemyId ]->intData( 4 , i );
					int freq = mEnemyDatas[ EnemyId ]->intData( 5 , i );
					bool endless = ( time == 0 ) ? true : false;
					ret->mSMPList.push_back( shared_ptr< IShotMovePattern >( new SM_StrAcceToP( time , speed , acce , freq ) ) );
				}else if( smpData == "StrDeceToP" ){
					int time = mEnemyDatas[ EnemyId ]->intData( 2 , i );
					int speed = mEnemyDatas[ EnemyId ]->intData( 3 , i );
					int dece = mEnemyDatas[ EnemyId ]->intData( 4 , i );
					int freq = mEnemyDatas[ EnemyId ]->intData( 5 , i );
					bool endless = ( time == 0 ) ? true : false;
					ret->mSMPList.push_back( shared_ptr< IShotMovePattern >( new SM_StrDeceToP( time , speed , dece , freq ) ) );
				}else if( smpData == "Circle" ){
					int time = mEnemyDatas[ EnemyId ]->intData( 2 , i );
					int speed = mEnemyDatas[ EnemyId ]->intData( 3 , i );
					int num = mEnemyDatas[ EnemyId ]->intData( 4 , i );
					int freq = mEnemyDatas[ EnemyId ]->intData( 5 , i );
					bool endless = ( time == 0 ) ? true : false;
					ret->mSMPList.push_back( shared_ptr< IShotMovePattern >( new SM_Circle( time , speed , num , freq ) ) );
				}else if( smpData == "Random" ){
					int time = mEnemyDatas[ EnemyId ]->intData( 2 , i );
					int speed = mEnemyDatas[ EnemyId ]->intData( 3 , i );
					int num = mEnemyDatas[ EnemyId ]->intData( 4 , i );
					int freq = mEnemyDatas[ EnemyId ]->intData( 5 , i );
					bool endless = ( time == 0 ) ? true : false;
					ret->mSMPList.push_back( shared_ptr< IShotMovePattern >( new SM_Random( time , speed , num ,  freq ) ) );
				}else if( smpData == "RandomAcce" ){
					int time = mEnemyDatas[ EnemyId ]->intData( 2 , i );
					int speed = mEnemyDatas[ EnemyId ]->intData( 3 , i );
					int num = mEnemyDatas[ EnemyId ]->intData( 4 , i );
					int acce = mEnemyDatas[ EnemyId ]->intData( 5 , i );
					int freq = mEnemyDatas[ EnemyId ]->intData( 6 , i );
					bool endless = ( time == 0 ) ? true : false;
					ret->mSMPList.push_back( shared_ptr< IShotMovePattern >( new SM_RandomAcce( time , speed , num , acce , freq ) ) );
				}else if( smpData == "RandomDece" ){
					int time = mEnemyDatas[ EnemyId ]->intData( 2 , i );
					int speed = mEnemyDatas[ EnemyId ]->intData( 3 , i );
					int num = mEnemyDatas[ EnemyId ]->intData( 4 , i );
					int dece = mEnemyDatas[ EnemyId ]->intData( 5 , i );
					int freq = mEnemyDatas[ EnemyId ]->intData( 6 , i );
					bool endless = ( time == 0 ) ? true : false;
					ret->mSMPList.push_back( shared_ptr< IShotMovePattern >( new SM_RandomDece( time , speed ,  num , dece , freq ) ) );
				}else if( smpData == "Loop" ){
					ret->mSMPList.push_back( shared_ptr< IShotMovePattern >( new SM_Loop() ) );
				}else if( smpData == "Ereka" ){
					ret->mSMPList.push_back( shared_ptr< IShotMovePattern >( new SM_Ereka() ) );
				}
			}
		}
		return ret;
	}
	shared_ptr< EnemyData > CreateEnemyData( const int MapEnemyId ){
		shared_ptr< EnemyData > ret = shared_ptr< EnemyData >( new EnemyData() );
		const int endMap = mMapData->allLine();
		int id = -1;
		for( int i = 1 ; i < endMap ; ++i ){
			if( mMapData->intData( 0 , i ) == MapEnemyId ){
				id = mMapData->intData( 2 , i );
				ret = MapDataFactory::GetEnemyData( id );
				ret->mNo = mMapData->intData( 0 , i );
				ret->mTime = mMapData->intData( 1 , i );
				ret->mNotDeathTime = mMapData->intData( 5 , i );
				ret->mX = mMapData->intData( 3 , i );
				ret->mY = mMapData->intData( 4 , i );
				break;
			}
		}
		if( id == -1 ){
			return ret;
		}
		return ret;
	}
private:
	shared_ptr< CSVLoader > mMapData;
	std::vector< shared_ptr< CSVLoader > > mEnemyDatas;
	std::vector< shared_ptr< CSVLoader > > mShotDatas;
	std::map< std::string , shared_ptr< Image > > mImages;
};
class DeathEffectActor : public BaseActor{
private:
	 int mNum;//
	 int mCnt;
	 float mAngle[ 10 ];
	 int mColor[ 10 ];
	 int mIndex;
	 bool mSoundPlay;
	 SoundType mSoundType;
	 shared_ptr< Sound > mSound;
public:
	DeathEffectActor( Math::Vector& pos , int num , SoundType sound = SOUND_None ) : BaseActor( pos , ACT_Effect , 0 ){
		mNum = ( num < 11 ) ? num : 10;
		mCnt = 0;
		for( int i = 0 ; i < mNum ; ++i ){
			mAngle[ i ] = Math::degeeToRad( gApp->GetRandom( 0 , 361 ) );
			mColor[ i ] = gApp->GetRandom( 0 , 2 );
		}
		mIndex = 0;
		mSoundType = sound;
		switch( sound ){
			case SOUND_Bom:
				mSound =shared_ptr< Sound >( new Sound( gSoundDatas->mBom->handle() ) );
				break;
			case SOUND_Break:
				mSound = shared_ptr< Sound >( new Sound( gSoundDatas->mBreak->handle() ) );
				break;
			case SOUND_Hit:
				mSound = shared_ptr< Sound >( new Sound( gSoundDatas->mHit->handle() ) );
				break;
		}
		if( sound != SOUND_None ){ mSound->setVolume( gSoundDatas->mVolume ); }
		mSoundPlay = false;
	}
	~DeathEffectActor(){
	}
	void VOnUpdate( Game* game , int deltaMilliseconds ){
		++mCnt;
		if( mCnt % 30 == 0 ){
			++mIndex;
		}
		if( mIndex >= mNum ){
			mIsDeath = true;
		}
		if( mSoundType != SOUND_None && mSoundPlay == false ){
			mSound->play( Sound::BACK );
			mSoundPlay = true;
		}
	}
	void VOnDraw( Game* game , int deltaMilliseconds ){
		SetDrawBlendMode( DX_BLENDMODE_ALPHA , 255 / ( ( mCnt % 30 / 30 ) + 1 ) ); 
		if( mColor[ mIndex ] == 0 ){
			gShotPicts->mMagicCircleBlue->rotateDraw( mPos.x , mPos.y , mAngle[ mIndex ] , mCnt % 30 / 20.0f );
		}else if( mColor[ mIndex ] == 1 ){
			gShotPicts->mMagicCircleRed->rotateDraw( mPos.x , mPos.y , mAngle[ mIndex ] , mCnt % 30 / 20.0f );
		}
		SetDrawBlendMode( DX_BLENDMODE_NOBLEND , 0 );
	}
};
class PlayerShotActor : public BaseActor , public IEventListener{
private:
	float mSpeed;
	int mPower;
public:
	PlayerShotActor( Math::Vector& pos , float speed , int power );
	~PlayerShotActor();
	int GetPower(){ return mPower; }
	void VOnUpdate( Game* game , int deltaMilliseconds );
	void VOnDraw( Game* game , int deltaMilliseconds );
	void VListen( IEventData* eventData );
};
class EnemyActor : public BaseActor , public IEventListener{
public:
	EnemyActor( shared_ptr< EnemyData > data );
	~EnemyActor();
	virtual void VOnUpdate( Game* game , int deltaMilliseconds );
	virtual void VOnDraw( Game* game , int deltaMilliseconds );
	virtual void VListen( IEventData* eventData );
	bool IsBoss(){
		return mData->mIsBoss;
	}
	int MaxHp() const{
		return mMaxHp;
	}
	int Hp() const{
		return mData->mLife;
	}
private:
	shared_ptr< EnemyData > mData;
	std::vector< shared_ptr< EnemyActionPattern > >::iterator mEAPItr;
	std::vector< shared_ptr< IShotMovePattern > >::iterator mSMPItr;
	unsigned int mCnt;
	int mMaxHp;

	shared_ptr< FTLib::Music::Sound > mDeathSound;
};
class PlayerActor : public BaseActor , public IEventListener{
private:
	int mLife;
	int mMuteki;
	int mShotCnt;
	bool mShift;
	Game* mGame;
	int mScore;
	int mPower;
public:
	PlayerActor( Game* game );
	~PlayerActor();
	int GetMuteki() const;
	int GetScore() const{ return mScore; }
	int GetPower() const{ return mPower; }
	int GetLife() const{ return mLife; }
	void AddScore( int point ){ mScore += point; mPointGetSound->play( Sound::BACK ); }
	void AddPower( int power ){ mPower += power; mPowerGetSound->play( Sound::BACK ); }
	void AddLife( int num ){ mLife += num; }
	void SetLife( int life ){ mLife = life; }
	virtual void VOnUpdate( Game* game , int deltaMilliseconds );
	virtual void VOnDraw( Game* game , int deltaMilliseconds );
	virtual void VListen( IEventData* eventData );
private:
	shared_ptr< Image > mShot;
	Pad* mPad;
	Keyboard* mKeyboard;
	shared_ptr< Image > mImage;
	shared_ptr< Sound > mPointGetSound;
	shared_ptr< Sound > mPowerGetSound;
	shared_ptr< Sound > mDeathSound;
};
class BaseShot : public BaseActor , public IEventListener{
public:
	BaseShot( IActor* enemy , int size ) : BaseActor( Math::getVector( enemy->VGetPos().x , enemy->VGetPos().y , 0 ) , ACT_EnemyShot , size ){};
	virtual ~BaseShot(){};
	void VListen( IEventData* eventData ){
		if( eventData->VGetType() == ET_Hit ){
			EVE_Hit* hitEvent = dynamic_cast< EVE_Hit* >( eventData );
			IActor* Obj1 = hitEvent->Obj1();
			IActor* Obj2 = hitEvent->Obj2();
			if( Obj1 == this ){
				if( Obj2->VGetType() == ACT_Player ||
					Obj2->VGetType() == ACT_PlayerBlade )
				{
					mIsDeath = true;
				}
			}
			if( Obj2 == this ){
				if( Obj1->VGetType() == ACT_Player ||
					Obj1->VGetType() == ACT_PlayerBlade )
				{
					mIsDeath = true;
				}
			}
		}
	}
protected:
	void DeathCheck(){	
		if( mPos.x < -20.0f || mPos.x > 470 || mPos.y < -30.0f || mPos.y > 620.0f ){
			mIsDeath = true;
		}
	}
};
class StraightShot : public BaseShot{
private:
	float mAngle;
	float mSpeed;
public:
	StraightShot( IActor* enemy ,  float angle , float speed ) : BaseShot( enemy , 2 ){
		mAngle = angle;
		mSpeed = speed;
	}
	virtual void VOnUpdate( Game* game , int deltaMilliseconds ){
		mPos.x += cosf( mAngle ) * mSpeed;
		mPos.y += sinf( mAngle ) * mSpeed;
	}
	void VOnDraw( Game* game , int deltaMilliseconds ){
		gShotPicts->mPinkShot->rotateDraw(
			mPos.x,
			mPos.y,
			mAngle );
		DeathCheck();
	}
};
class TransShot : public BaseShot{
private:
	float mAngle;
	float mSpeed;
	float mTrans;
public:
	TransShot( IActor* enemy , float angle , float speed , float trans ) : BaseShot( enemy , 2 ){
		mAngle = angle;
		mSpeed = speed;
		mTrans = trans;
	}
	virtual void VOnUpdate( Game* game , int deltaMilliseconds ){
		mPos.x += cosf( mAngle ) * mSpeed;
		mPos.y += sinf( mAngle ) * mSpeed;
		mSpeed += mTrans;
	}
	virtual void VOnDraw( Game* game , int deltaMilliseconds ){
		gShotPicts->mNormalShot->draw( mPos.x - gShotPicts->mNormalShot->width() / 2,
			mPos.y - gShotPicts->mNormalShot->height() / 2 );
		DeathCheck();
	}
};
class RotateShot : public BaseShot{
private:
	float mAngle;
	float mSpeed;
	float mRotate;
public:
	RotateShot( IActor* enemy , float angle , float speed , float rotate ) : BaseShot( enemy , 2 ){
		mAngle = angle;
		mSpeed = speed;
		mRotate = rotate;
	}
	virtual void VOnUpdate( Game* game , int delataMilliseconds ){
		mPos.x += cosf( mAngle ) * mSpeed;
		mPos.y += sinf( mAngle ) * mSpeed;
		mAngle += mRotate;
	}
	virtual void VOnDraw( Game* game , int deltaMilliseconds ){
		gShotPicts->mNormalShot->draw( mPos.x - gShotPicts->mNormalShot->width() / 2,
			mPos.y - gShotPicts->mNormalShot->height() / 2 );
		DeathCheck();
	}
};

class DelayRotateShot : public BaseShot{
private:
	float mAngle;
	float mSpeed;
	float mRotate;
	int mDelay;
	int mCnt;
public:
	DelayRotateShot( IActor* enemy , float angle , float speed , float rotate , int delay ) : BaseShot( enemy , 2 ){
		mAngle = angle;
		mSpeed = speed;
		mRotate = rotate;
		mDelay = delay;
		mCnt = 0;
	}
	virtual void VOnUpdate( Game* game , int delataMilliseconds ){
		mPos.x += cosf( mAngle ) * mSpeed;
		mPos.y += sinf( mAngle ) * mSpeed;
		if( mCnt < mDelay ){
			mAngle += mRotate;
			++mCnt;
		}
		DeathCheck();
	}
	virtual void VOnDraw( Game* game , int deltaMilliseconds ){
		gShotPicts->mNormalShot->draw( mPos.x - gShotPicts->mNormalShot->width() / 2,
			mPos.y - gShotPicts->mNormalShot->height() / 2 );
	}
};
class ErekaLineShot : public BaseShot{
private:
	int mDelay;
	int mCnt;
	float mAngle;
	float m_Speed;
	float mSpeed;
public:
	ErekaLineShot( IActor* enemy , int delay , float speed , Math::Vector& pos ) : BaseShot( enemy , 2 ){
		mDelay = delay;
		mPos = pos;
		mCnt = 0;
		mAngle = 0;
		m_Speed = speed;
		mSpeed = 0;
	}
	virtual void VOnUpdate( Game* game , int deltaMilliseconds );
	virtual void VOnDraw( Game* game , int deltaMilliseconds );
};
class DelayStraightShot : public BaseShot{
private:
	int mDelay;
	int mCnt;
	float mAngle;
	float m_Speed;
	float mSpeed;
public:
	DelayStraightShot( IActor* enemy , int delay , float speed , float angle , Math::Vector& pos ) : BaseShot( enemy , 2 ){
		mDelay = delay;
		mPos = pos;
		mCnt = 0;
		mAngle = angle;
		m_Speed = speed;
		mSpeed = 0;
	}
	virtual void VOnUpdate( Game* game , int deltaMilliseconds ){
		if( mCnt == mDelay ){
			mSpeed = m_Speed;
		}
		mPos.x += cosf( mAngle ) * mSpeed;
		mPos.y += sinf( mAngle ) * mSpeed;
		DeathCheck();
		++mCnt;
	}
	virtual void VOnDraw( Game* game , int deltaMilliseconds ){
		gShotPicts->mStarBlueShot->rotateDraw(
			mPos.x,
			mPos.y,
			mAngle );
	}
};
class BaseItemActor : public BaseActor , public IEventListener{
protected:
	int mObjSize;
	int mSpeed;
	bool mGuided;
public:
	BaseItemActor( Math::Vector& pos , int size , int speed ) : BaseActor( pos , ACT_Item , 72 ){
		mObjSize = size;
		mSpeed = speed;
		mGuided = false;
	}
	virtual ~BaseItemActor(){}
	void VOnUpdate( Game* game , int deltaMilliseconds );
	virtual void Effect( PlayerActor* player ) = 0;
	void VListen( IEventData* eventData ){
		if( eventData->VGetType() == ET_Hit ){
			EVE_Hit* hitEvent = dynamic_cast< EVE_Hit* >( eventData );
			IActor* Obj1 = hitEvent->Obj1();
			IActor* Obj2 = hitEvent->Obj2();
			IActor* Player = 0;
			if( Obj1 == this && Obj2->VGetType() == ACT_Player ){
				Player = Obj2;
			}
			if( Obj1->VGetType() == ACT_Player && Obj2 == this ){
				Player = Obj1;
			}
			if( Player != NULL ){
				mHitCircle = mObjSize;
				if( mGuided ){
					Effect( dynamic_cast< PlayerActor* >( Player ) );
					mIsDeath = true;
				}
				mGuided = true;
			}
		}
	}
};
class PointItemActor : public BaseItemActor{
private:
	int mPoint;
public:
	PointItemActor( Math::Vector& pos , int point ) : BaseItemActor( pos , 24 , 4 )
	{
		mPoint = point;
	}
	~PointItemActor(){
	}
	void VOnDraw( Game* game , int deltaMilliSeconds ){
		gShotPicts->mPointItem->draw(
			mPos.x - gShotPicts->mPointItem->width() / 2,
			mPos.y - gShotPicts->mPointItem->height() / 2 );
	}
	void Effect( PlayerActor* player ){
		player->AddScore( mPoint );
	}
};
class PowerItemActor : public BaseItemActor{
private:
	int mPower;
public:
	PowerItemActor( Math::Vector& pos , int power ) : BaseItemActor( pos , 24 , 4 ){
		mPower = power;
	}
	~PowerItemActor(){
	}
	void VOnDraw( Game* game , int deltaMilliSeconds ){
		gShotPicts->mPowerItem->draw(
			mPos.x - gShotPicts->mPowerItem->width() / 2,
			mPos.y - gShotPicts->mPowerItem->height() / 2 );
	}
	void Effect( PlayerActor* player ){
		player->AddPower( mPower );
	}
};
class LifeItemActor : public BaseItemActor{
private:
	int mLife;
public:
	LifeItemActor( Math::Vector& pos , int life ) : BaseItemActor( pos , 8 , 4 ){
		mLife = life;
	}
	~LifeItemActor(){
	}
	void VOnDraw( Game* game , int deltaMilliSeconds ){
		gShotPicts->mLifeItem->draw(
			mPos.x - gShotPicts->mLifeItem->width() / 2,
			mPos.y - gShotPicts->mLifeItem->height() / 2 );
	}
	void Effect( PlayerActor* player ){
		player->AddLife( mLife );
	}
};