#pragma once
#include "Std.h"
#include "Actors.h"
class Game : public ISequence , IEventListener{
private:
	enum GameSequence{
		GAMSEQ_Game,
		GAMSEQ_Menu,
		GAMSEQ_Clear,
		GAMSEQ_GameOver,
		GAMSEQ_None
	};
	MapData mMapData;
	int mTime;
	shared_ptr< Image > mScoreBg;
	shared_ptr< Image > mUnderBarBg;
	int mScore;
	bool mInit;
	typedef std::vector< shared_ptr< IActor > > Actors;
	shared_ptr< PlayerActor > mPlayer;
	Actors mEnemys;
	Actors mPlayerShots;
	Actors mEnemyShots;
	Actors mEffects;
	Actors mItems;
	Pad* mPad;
	Keyboard* mKeyboard;
	GameSequence mGameSequence;
	int mMenuIndex;
	shared_ptr< Image > mMenuPict;
	shared_ptr< Image > mQuitPict;
	shared_ptr< Image > mToFirstPict;
	shared_ptr< Image > mToGamePict;
	shared_ptr< Image > mVolumeBarPict;
	EnemyActor* mBoss;
	shared_ptr< Image > mClearPict;
	shared_ptr< Image > mGameOverPict;
	shared_ptr< Image > mLifeIcon;
	shared_ptr< Font > mStoryFont;
	shared_ptr< Font > mScoreFont;
	int mClearCnt;
	int mDeathCnt;
	int mBgCnt;
	int mBgHeight;
	int mVolume;
	int mStoryIndex;
	bool mStoryStart;
	bool mStoryEnd;
	bool mClearEnemyShot;
	ItemType mConversionItemType;
	int mConversionItemPoint;
	int mHiScore;
public:
	Game(){
		MapDataFactory mdf( "Data/Game/Map.txt" );
		mdf.setEnemyData( "Data/Game/EnemyList.txt" );
		mMapData = mdf.GetMapData();
		mScoreBg = shared_ptr< Image >( new Image( "Pict/Game/ScoreBg.png" ) );
		mUnderBarBg = shared_ptr< Image >( new Image( "Pict/Game/UnderBar.png" ) );
		mTime = 0;
		mInit = false;
		gShotPicts = new ShotPicts();
		gSoundDatas = new SoundDatas();
		mPad = Manager::instance()->getPad( 0 );
		mKeyboard = Manager::instance()->getKeyboard();
		mGameSequence = GAMSEQ_Game;
		mMenuIndex = 0;
		mMenuPict = shared_ptr< Image >( new Image( "Pict/Game/Menu.png" ) );
		mQuitPict = shared_ptr< Image >( new Image( "Pict/Game/Quit.png" ) );
		mToFirstPict = shared_ptr< Image >( new Image( "Pict/Game/ToFirst.png" ) );
		mToGamePict = shared_ptr< Image >( new Image( "Pict/Game/ToGame.png" ) );
		mVolumeBarPict = shared_ptr< Image >( new Image( "Pict/Game/VolumeBar.png" ) );
		mLifeIcon = shared_ptr< Image >( new Image( "Pict/Game/LifeIcon.png" ) );
		mBoss = 0;
		mClearCnt = 0;
		mDeathCnt = 0;
		mBgCnt = 0;
		mBgHeight = mMapData.mBg->height();
		mVolume = 128;
		mMapData.mRoadSound->setVolume( mVolume );
		mMapData.mRoadSound->play( Sound::LOOP );
		mStoryIndex = 0;
		mStoryStart = false;
		mStoryEnd = false;
		mStoryFont = shared_ptr< Font >( new Font() );
		mScoreFont = shared_ptr< Font >( new Font( "" , 36 , Font::AA_EDGE ) );
		mClearEnemyShot = false;
		mConversionItemType = ITEM_None;
		mConversionItemPoint = 0;
		CSVLoader csvl( "save.txt" );
		if( csvl.allLine() != 0 ){
			mVolume = csvl.intData( 2 , 0 );
			gSoundDatas->mVolume = mVolume;
			mHiScore = csvl.intData( 2 , 1 );
		}
	}
	~Game(){
		SAFE_DELETE( gShotPicts );
		SAFE_DELETE( gSoundDatas );
		Save( mVolume , mHiScore );
	}
	void Save( int volume , int hiScore ){
		CSVWriter csvw;
		csvw.setIntData( 0 , 0 , 0 );
		csvw.setStrData( 1 , 0 , "Volume" );
		csvw.setIntData( 2 , 0 , volume );
		csvw.setIntData( 0 , 1 , 1 );
		csvw.setStrData( 1 , 1 , "HiScore" );
		csvw.setIntData( 2 , 1 , hiScore );
		csvw.write( "save.txt" );
	}
	void OnUpdate( IrisBaseGame* game , unsigned int elpsedFrame ){
		if( mGameSequence == GAMSEQ_Menu ){
			MenuDraw( game );
			//sukoanohyouji
			mScoreBg->draw( 450 , 0 );
			mUnderBarBg->draw( 0 , 560 );
			for( int i = 0 ; i < mPlayer->GetLife() ; ++i ){
				mLifeIcon->draw( 570 + i * 22 , 52 );
			}
			mScoreFont->draw( boost::lexical_cast< std::string >( mPlayer->GetScore() ).c_str() , 580 , 92 , 236 , 20 , 20 );
			mScoreFont->draw( boost::lexical_cast< std::string >( mPlayer->GetPower() ).c_str() , 580 , 152 , 20 , 236 , 20 );
		}else if( mGameSequence == GAMSEQ_Game ){
			GameUpdate( game , elpsedFrame );
		
			//sukoanohyouji
			mScoreBg->draw( 450 , 0 );
			mUnderBarBg->draw( 0 , 560 );
			for( int i = 0 ; i < mPlayer->GetLife() ; ++i ){
				mLifeIcon->draw( 570 + i * 22 , 52 );
			}
			if( mBoss ){
				float bossX = mBoss->VGetPos().x;
				DrawBox( bossX - 5 , 560 , bossX + 5 , 600 , GetColor( 36 , 36 , 255 ) , 1 );
				mStoryFont->centeringDraw( "BOSS" , bossX , 570 , 0 , 0 , 255 , 255 , 255 , 255 );
			}
			mScoreFont->draw( boost::lexical_cast< std::string >( mPlayer->GetScore() ).c_str() , 580 , 92 , 236 , 20 , 20 );
			mScoreFont->draw( boost::lexical_cast< std::string >( mPlayer->GetPower() ).c_str() , 580 , 152 , 20 , 236 , 20 );
		}else if( mGameSequence == GAMSEQ_Clear ){
			GameClearDraw( game );
		}else if( mGameSequence == GAMSEQ_GameOver ){
			GameOverDraw( game );
		}
	}
	void GameUpdate( IrisBaseGame* game , unsigned int elpsedFrame ){
		//
		if( !mInit ){ Init( game ); }
		//G̒ǉ[`
		std::vector< shared_ptr< EnemyData > >::iterator i = mMapData.mEnemyDatas.begin();
		for( i ; i != mMapData.mEnemyDatas.end() ; ){
			if( mTime < ( *i )->mTime ){
				break;
			}
			if( mTime == ( *i )->mTime ){
				shared_ptr< EnemyActor > enemy = shared_ptr< EnemyActor >( new EnemyActor( ( *i ) ) );
				if( enemy->IsBoss() ){
					mBoss = enemy.get();
				}
				mEnemys.push_back( enemy );
				if( ( *i )->mNo == mMapData.mStory.mBossNo ){
					mStoryStart = true;
				}
				i = mMapData.mEnemyDatas.erase( i );
				continue;
			}
			++i;
		}
		//wi
		++mBgCnt;
		if( mBgHeight != NULL ){
			mMapData.mBg->draw( 0 , mBgCnt % mBgHeight - mBgHeight ); 
			mMapData.mBg->draw( 0 , mBgCnt % mBgHeight );
		}
		//Abvf[g
		//GeAeAGA
		bool update = true;
		if( mStoryStart && !mStoryEnd ){
			update = false;
		}
		Actors::iterator itr = mEffects.begin();
		while( itr != mEffects.end() ){
			if( update ){
				( *itr )->VOnUpdate( this , elpsedFrame );
			}
			( *itr )->VOnDraw( this , elpsedFrame );
			++itr;
		}
		itr = mItems.begin();
		while( itr != mItems.end() ){
			if( update ){
				( *itr )->VOnUpdate( this , elpsedFrame );
			}
			( *itr )->VOnDraw( this , elpsedFrame );
			++itr;
		}
		itr = mEnemys.begin();
		while( itr != mEnemys.end() ){
			if( update ){
				( *itr )->VOnUpdate( this , elpsedFrame );
			}
			( *itr )->VOnDraw( this , elpsedFrame );
			++itr;
		}
		itr = mPlayerShots.begin();
		while( itr != mPlayerShots.end() ){
			( *itr )->VOnUpdate( this , elpsedFrame );
			( *itr )->VOnDraw( this , elpsedFrame );
			++itr;
		}
		mPlayer->VOnDraw( this , elpsedFrame );
		itr = mEnemyShots.begin();
		while( itr != mEnemyShots.end() ){
			if( update ){
				( *itr )->VOnUpdate( this , elpsedFrame );
			}
			( *itr )->VOnDraw( this , elpsedFrame );
			++itr;
		}
		//fobO
		static bool HIT_ON = true;
		if( Manager::instance()->getPad( 0 )->isOn( Pad::BUT_X ) ){
			HIT_ON = !HIT_ON;
		}
				//vC[SmF
		if( mPlayer->GetLife() <= 0 ){
			++mDeathCnt;
			Display::instance()->setDrawBright( 255 - mDeathCnt , 255 - mDeathCnt * 2 , 255 - mDeathCnt * 2 );
			if( mDeathCnt > 127 ){
				Display::instance()->setDrawBright( 255 , 255 , 255 );
				mGameOverPict = shared_ptr< Image >( new Image( "Pict/Game/GameOver.png" ) );
				mGameSequence = GAMSEQ_GameOver;
			}
		}else{//łȂ
			mPlayer->VOnUpdate( this , elpsedFrame );
			if( mPad->isDown( Pad::BUT_START ) ||
				mKeyboard->isDown( Keyboard::KEY_ESC ) )
			{
				Display::instance()->setDrawBright( 255  , 255 , 255 );
				mMenuIndex = 0;
				mGameSequence = GAMSEQ_Menu;
			}		
			if( HIT_ON && update ){ HitCheck(); }
		}
		//{XłȂ@{X[h
		if( mBoss != NULL ){
			if( mMapData.mRoadSound->isPlay() ){
				mMapData.mRoadSound->stop();
			}
			if( !mMapData.mBossSound->isPlay() ){
				mMapData.mBossSound->setVolume( mVolume );
				mMapData.mBossSound->play( Sound::LOOP );
			}
			SetDrawBlendMode( DX_BLENDMODE_ALPHA , 180 );
			DrawBox( 10 , 10 , 440 , 30 , GetColor( 128 , 128 , 128 ) , 1 );
			DrawBox( 10 , 10 , mBoss->Hp() * 420 / mBoss->MaxHp() + 10 , 30 , GetColor( 255 , 255 , 255 ) , 1 );
			SetDrawBlendMode( DX_BLENDMODE_NOBLEND , 0 );
			if( mBoss->VIsDeath() ){
				mBoss = NULL;
			}
		}else{
			if( mMapData.mBossSound->isPlay() ){
				mMapData.mBossSound->stop();
			}
			if( !mMapData.mRoadSound->isPlay() ){
				mMapData.mRoadSound->setVolume( mVolume );
				mMapData.mRoadSound->play( Sound::LOOP );
			}
		}
		if( mClearEnemyShot ){
			itr = mEnemyShots.begin();
			while( itr != mEnemyShots.end() ){
				Math::Vector pos = ( *itr )->VGetPos();
				AddEffect( shared_ptr< IActor >( new DeathEffectActor( pos , 1 , SOUND_None ) ) );
				itr = mEnemyShots.erase( itr );
				continue;
			}
			mClearEnemyShot = false;
		}
		if( mConversionItemType != ITEM_None ){
			itr = mEnemyShots.begin();
			switch( mConversionItemType ){
				case ITEM_Point:
					while( itr != mEnemyShots.end() ){
						Math::Vector pos = ( *itr )->VGetPos();
						AddItem( shared_ptr< IActor >( new PointItemActor( pos , mConversionItemPoint ) ) );
						itr = mEnemyShots.erase( itr );
						continue;
					}
					break;
				case ITEM_Power:
					while( itr != mEnemyShots.end() ){
						Math::Vector pos = ( *itr )->VGetPos();
						AddItem( shared_ptr< IActor >( new PowerItemActor( pos , mConversionItemPoint ) ) );
						itr = mEnemyShots.erase( itr );
						continue;
					}
					break;
				case ITEM_Life:
					while( itr != mEnemyShots.end() ){
						Math::Vector pos = ( *itr )->VGetPos();
						AddItem( shared_ptr< IActor >( new LifeItemActor( pos , mConversionItemPoint ) ) );
						itr = mEnemyShots.erase( itr );
						continue;
					}
					break;
			}
			mConversionItemType = ITEM_None;
		}

		itr = mEffects.begin();
		while( itr != mEffects.end() ){
			if( ( *itr )->VIsDeath() ){
				itr = mEffects.erase( itr );
				continue;
			}
			++itr;
		}
		itr = mItems.begin();
		while( itr != mItems.end() ){
			if( ( *itr )->VIsDeath() ){
				itr = mItems.erase( itr );
				continue;
			}
			++itr;
		}
		itr = mEnemyShots.begin();
		while( itr != mEnemyShots.end() ){
			if( ( *itr )->VIsDeath() ){
				itr = mEnemyShots.erase( itr );
				continue;
			}
			++itr;
		}
		itr = mEnemys.begin();
		while( itr != mEnemys.end() ){
			if( ( *itr )->VIsDeath() ){
				itr = mEnemys.erase( itr );
				continue;
			}
			++itr;
		}
		itr = mPlayerShots.begin();
		while( itr != mPlayerShots.end() ){
			if( ( *itr )->VIsDeath() ){
				itr = mPlayerShots.erase( itr );
				continue;
			}
			++itr;
		}
		if( mBoss == NULL ){
			++mTime;
		}
		if( !update ){
			StoryDraw( game );
		}
		//NA
		if( mMapData.mEnemyDatas.size() == 0 && mBoss == 0 ){
			++mClearCnt;
			if( mClearCnt == 120 ){
				AddItem( shared_ptr< IActor >( new PointItemActor( Math::getVector( 225 , 200 , 0 ) , mPlayer->GetLife() * 10000 ) ) );
			}
			if( mClearCnt > 540 ){
				mClearPict = shared_ptr< Image >( new Image( "Pict/Game/Clear.png" ) );
				mGameSequence = GAMSEQ_Clear;
			}
		}
	}
	void StoryDraw( IrisBaseGame* game ){
		StoryData data = mMapData.mStory;
		std::map< std::string , StoryChara >::iterator itr = data.mCharaMap.begin();
		while( itr != data.mCharaMap.end() ){
			bool bright = false;
			if( ( *itr ).first == data.mLines[ mStoryIndex ].mName ){
				bright = true;
			}
			int x = 0;//Ȃ80@EȂ370
			if( ( *itr ).second.mIsLeft ){
				x = 80;
			}else{
				x = 370;
			}
			shared_ptr< Image > img = ( *itr ).second.mImage[ data.mLines[ mStoryIndex ].mImageNo - 1 ];
			if( !bright ){
				SetDrawBright( 128 , 128 , 128 );
			}
			img->draw( x - img->width() / 2 , 560 - img->height() );
			SetDrawBright( 255 , 255 , 255 );
			++itr;
		}
		SetDrawBlendMode( DX_BLENDMODE_ALPHA , 128 );
		DrawBox( 10 , 460 , 440 , 550 , GetColor( 32 , 32 , 32 ) , 1 );
		SetDrawBlendMode( DX_BLENDMODE_NOBLEND , 0 );
		mStoryFont->draw( data.mLines[ mStoryIndex ].mName.c_str() , 20 , 470 , 232 , 0 , 0 );
		mStoryFont->rectDraw( data.mLines[ mStoryIndex ].mText.c_str() , 20 , 490 , 410 , 160 , 255 , 255 , 255 , 128 , 64 , 64 );
		if( mPad->isDown( Pad::BUT_A ) ||
			mKeyboard->isDown( Keyboard::KEY_Z ) ){
				++mStoryIndex;
		}
		if( mStoryIndex >= data.mLines.size() ){
			mStoryEnd = true;
		}
	}
	void MenuDraw( IrisBaseGame* game ){
		//Q[ɖ߂A蒼AI
		if( mMenuIndex > 0 ){
			if( mPad->isDown( Pad::BUT_UP ) ||
				mKeyboard->isDown( Keyboard::KEY_UP ) )
			{
				--mMenuIndex;
			}
		}
		if( mMenuIndex < 2 ){
			if( mPad->isDown( Pad::BUT_DOWN ) ||
				mKeyboard->isDown( Keyboard::KEY_DOWN ) )
			{
				++mMenuIndex;
			}
		}
		if( mPad->isOn( Pad::BUT_LEFT ) ||
			mKeyboard->isOn( Keyboard::KEY_LEFT ) )
		{
			if( mVolume > 0 ){
				--mVolume;
				--gSoundDatas->mVolume;
				mMapData.mBossSound->setVolume( mVolume );
				mMapData.mRoadSound->setVolume( mVolume );
			}
		}
		if( mPad->isOn( Pad::BUT_RIGHT ) ||
			mKeyboard->isOn( Keyboard::KEY_RIGHT ) )
		{
			if( mVolume < 255 ){
				++mVolume;
				++gSoundDatas->mVolume;
				mMapData.mBossSound->setVolume( mVolume );
				mMapData.mRoadSound->setVolume( mVolume );
			}
		}
		if( mPad->isDown( Pad::BUT_A ) ||
			mKeyboard->isDown( Keyboard::KEY_Z ) )
		{
			switch( mMenuIndex ){
				case 0://Q[ɖ߂
					mGameSequence = GAMSEQ_Game;
					break;
				case 1://蒼
					game->SequenceMove( SEQ_Loading );
					break;
				case 2://I
					game->SequenceMove( SEQ_Quit );
					break;
			}
		}	
		mMenuPict->draw( 0 , 0 );
		switch( mMenuIndex ){
			case 0://Q[ɖ߂
				mToGamePict->draw( 70 , 70 );
				break;
			case 1://蒼
				mToFirstPict->draw( 70 , 70 + 60 );
				break;
			case 2://I
				mQuitPict->draw( 70 , 70 + 120 );
				break;
		}
		mVolumeBarPict->rectDraw( 40 , 320 , 0 , 0 , mVolume * mVolumeBarPict->width() / 255 , mVolumeBarPict->height() );
	}
	void GameClearDraw( IrisBaseGame* game ){
		if( mPad->isDown( Pad::BUT_A ) ||
			mKeyboard->isDown( Keyboard::KEY_Z ) )
		{
			if( mPlayer->GetScore() > mHiScore ){
				mHiScore = mPlayer->GetScore();
			}
			Save( mVolume , mHiScore );
			if( mMapData.mNextStageName == "End" ){
				game->SequenceMove( SEQ_Opening );
			}else{
				std::string mapPath = "Data/Game/";
				mapPath.append( mMapData.mNextStageName );
				MapDataFactory mdf = MapDataFactory( mapPath.c_str() );
				mdf.setEnemyData( "Data/Game/EnemyList.txt" );
				mMapData = mdf.GetMapData();
				mGameSequence = GAMSEQ_Game;
				mTime = 0;
				mMenuIndex = 0;
				mBoss = 0;
				mEnemys.clear();
				mPlayerShots.clear();
				mEnemyShots.clear();
				mClearCnt = 0;
				mDeathCnt = 0;
				mBgCnt = 0;
				mStoryIndex = 0;
				mStoryStart = false;
				mStoryEnd = false;
				mClearEnemyShot = false;
				mConversionItemType = ITEM_None;
				mConversionItemPoint = 0;
				mPlayer->VSetPos( Math::getVector( 225 , 480 , 0 ) );
			}
		} 
		mClearPict->draw( 0 , 0 );
		mScoreFont->draw(
			std::string( "Score : " ).append(
				boost::lexical_cast< std::string >( mPlayer->GetScore() ) ).append(
					"\nHiScore : " ).append(
						boost::lexical_cast< std::string >(
							mHiScore ) ).c_str() , 0 , 0 , 236 , 0 , 0 );
	}
	void GameOverDraw( IrisBaseGame* game ){
		if( mPad->isDown( Pad::BUT_A ) ||
			mKeyboard->isDown( Keyboard::KEY_Z ) )
		{
			if( mPlayer->GetScore() > mHiScore ){
				mHiScore = mPlayer->GetScore();
			}
			Save( mVolume , mHiScore );
			game->SequenceMove( SEQ_Opening );
		}
		mGameOverPict->draw( 0 , 0 );
		mScoreFont->draw( std::string( "Score : " ).append( boost::lexical_cast< std::string >( mPlayer->GetScore() ) ).c_str() , 0 , 0 , 236 , 0 , 0 );
	}
	void ClearEnemyShot(){
		mClearEnemyShot = true;
	}
	void EnemyShotConversionItem( ItemType type , int point ){
		mConversionItemType = type;
		mConversionItemPoint = point;
	}
	void Init( IrisBaseGame* game ){
		mPlayer = shared_ptr< PlayerActor >( new PlayerActor( this ) );
		mPlayer->SetLife( 4 );
		mInit = true;
	}
	void AddEnemyShot( shared_ptr< IActor > shot ){
		mEnemyShots.push_back( shot );
	}
	void AddPlayerShot( shared_ptr< IActor > shot ){
		mPlayerShots.push_back( shot );
	}
	void AddEffect( shared_ptr< IActor > effect ){
		mEffects.push_back( effect );
	}
	void AddItem( shared_ptr< IActor > item ){
		mItems.push_back( item );
	}
	void SetBoss( EnemyActor* boss ){
		mBoss = boss;
	}
	bool IsCollision( const Math::Vector& pos1 , int r1 , const Math::Vector& pos2 , int r2 ){
		if( ( pos1.x - pos2.x ) * ( pos1.x - pos2.x ) + ( pos1.y - pos2.y ) * ( pos1.y - pos2.y ) <= ( r1 + r2 ) * ( r1 + r2 ) ){
			return true;
		}
		return false;
	}
	void HitCheck(){
		Actors::iterator itr = mItems.begin();
		while( itr != mItems.end() ){
			if( IsCollision( mPlayer->VGetPos() , mPlayer->VGetHitCircle() , ( *itr )->VGetPos() , ( *itr )->VGetHitCircle() ) ){
				EVE_Hit* hitEvent = new EVE_Hit( mPlayer.get() , ( *itr ).get() );
				dynamic_cast< IEventListener* >( mPlayer.get() )->VListen( hitEvent );
				dynamic_cast< IEventListener* >( ( *itr ).get() )->VListen( hitEvent );
				SAFE_DELETE( hitEvent );
			}
			++itr;
		}
		itr = mEnemyShots.begin();
		while( itr != mEnemyShots.end() ){
			if( IsCollision( mPlayer->VGetPos() , mPlayer->VGetHitCircle() , ( *itr )->VGetPos() , ( *itr )->VGetHitCircle() ) ){
				EVE_Hit* hitEvent = new EVE_Hit( mPlayer.get() , ( *itr ).get() );
				dynamic_cast< IEventListener* >( mPlayer.get() )->VListen( hitEvent );
				dynamic_cast< IEventListener* >( ( *itr ).get() )->VListen( hitEvent );
				SAFE_DELETE( hitEvent );
			}
			++itr;
		}
		itr = mEnemys.begin();
		while( itr != mEnemys.end() ){
			if( IsCollision( mPlayer->VGetPos() , mPlayer->VGetHitCircle() , ( *itr )->VGetPos() , ( *itr )->VGetHitCircle() ) ){
				EVE_Hit* hitEvent = new EVE_Hit( mPlayer.get() , ( *itr ).get() );
				dynamic_cast< IEventListener* >( mPlayer.get() )->VListen( hitEvent );
				dynamic_cast< IEventListener* >( ( *itr ).get() )->VListen( hitEvent );
				SAFE_DELETE( hitEvent );
			}
			++itr;
		}
		itr = mPlayerShots.begin();
		while( itr != mPlayerShots.end() ){
			Actors::iterator EItr = mEnemys.begin();
			while( EItr != mEnemys.end() ){
				if( IsCollision( ( *itr )->VGetPos() , ( *itr )->VGetHitCircle() , ( *EItr )->VGetPos() , ( *EItr )->VGetHitCircle() ) ){
					EVE_Hit* hitEvent = new EVE_Hit( ( *itr ).get() , ( *EItr ).get() );
					dynamic_cast< IEventListener* >( ( *itr ).get() )->VListen( hitEvent );
					dynamic_cast< IEventListener* >( ( *EItr ).get() )->VListen( hitEvent );
					SAFE_DELETE( hitEvent );
				}
				++EItr;
			}
			++itr;
		}
	}
	/*
	void HitCheck(){

		Actors::iterator itr = mItems.begin();
		float leftP = mPlayer->VGetPos().x - mPlayer->VGetHitCircle();
		float rightP = mPlayer->VGetPos().x + mPlayer->VGetHitCircle();
		float topP = mPlayer->VGetPos().y - mPlayer->VGetHitCircle();
		float bottomP = mPlayer->VGetPos().y + mPlayer->VGetHitCircle();
		while( itr != mItems.end() ){
			float leftI = ( *itr )->VGetPos().x - ( *itr )->VGetHitCircle();
			float rightI = ( *itr )->VGetPos().x + ( *itr )->VGetHitCircle();
			float topI = ( *itr )->VGetPos().y - ( *itr )->VGetHitCircle();
			float bottomI = ( *itr )->VGetPos().y + ( *itr )->VGetHitCircle();
			if( ( leftP < rightI ) &&
				( rightP > leftI ) &&
				( topP < bottomI ) &&
				( bottomP < topI ) )
			{
				EVE_Hit* hitEvent = new EVE_Hit( mPlayer.get() , ( *itr ).get() );
				dynamic_cast< IEventListener* >( mPlayer.get() )->VListen( hitEvent );
				dynamic_cast< IEventListener* >( ( *itr ).get() )->VListen( hitEvent );
			}
			++itr;
		}
		itr = mEnemyShots.begin();
		while( itr != mEnemyShots.end() ){
			float leftE = ( *itr )->VGetPos().x - ( *itr )->VGetHitCircle();
			float rightE = ( *itr )->VGetPos().x + ( *itr )->VGetHitCircle();
			float topE = ( *itr )->VGetPos().y - ( *itr )->VGetHitCircle();
			float bottomE = ( *itr )->VGetPos().y + ( *itr )->VGetHitCircle();
			if( ( leftP < rightE ) &&
				( rightP > leftE ) &&
				( topP < bottomE ) &&
				( bottomP > topE ) )
			{
				EVE_Hit* hitEvent = new EVE_Hit( mPlayer.get() , ( *itr ).get() );
				dynamic_cast< IEventListener* >( mPlayer.get() )->VListen( hitEvent );
				dynamic_cast< IEventListener* >( ( *itr ).get() )->VListen( hitEvent );
			}
			++itr;
		}
		itr = mEnemys.begin();
		while( itr != mEnemys.end() ){
			float leftE = ( *itr )->VGetPos().x - ( *itr )->VGetHitCircle();
			float rightE = ( *itr )->VGetPos().x + ( *itr )->VGetHitCircle();
			float topE = ( *itr )->VGetPos().y - ( *itr )->VGetHitCircle();
			float bottomE = ( *itr )->VGetPos().y + ( *itr )->VGetHitCircle();
			if( ( leftP < rightE ) &&
				( rightP > leftE ) &&
				( topP < bottomE ) &&
				( bottomP > topE ) )
			{
				EVE_Hit* hitEvent = new EVE_Hit( mPlayer.get() , ( *itr ).get() );
				dynamic_cast< IEventListener* >( mPlayer.get() )->VListen( hitEvent );
				dynamic_cast< IEventListener* >( ( *itr ).get() )->VListen( hitEvent );
			}
			++itr;
		}
		itr = mPlayerShots.begin();
		while( itr != mPlayerShots.end() ){
			leftP = ( *itr )->VGetPos().x - ( *itr )->VGetHitCircle();
			rightP = ( *itr )->VGetPos().x + ( *itr )->VGetHitCircle();
			topP = ( *itr )->VGetPos().y - ( *itr )->VGetHitCircle();
			bottomP = ( *itr )->VGetPos().y + ( *itr )->VGetHitCircle(); 
			Actors::iterator Eitr = mEnemys.begin();
			while( Eitr != mEnemys.end() ){
				if( ( *itr )->VGetHitCircle() == 0 ){
					++Eitr;
					continue;
				}
				float leftE = ( *Eitr )->VGetPos().x - ( *Eitr )->VGetHitCircle();
				float rightE = ( *Eitr )->VGetPos().x + ( *Eitr )->VGetHitCircle();
				float topE = ( *Eitr )->VGetPos().y - ( *Eitr )->VGetHitCircle();
				float bottomE = ( *Eitr )->VGetPos().y + ( *Eitr )->VGetHitCircle();
				if( ( leftP < rightE ) &&
					( rightP > leftE ) &&
					( topP < bottomE ) &&
					( bottomP > topE ) )
				{
					EVE_Hit* hitEvent = new EVE_Hit( ( *itr ).get() , ( *Eitr ).get() );
					dynamic_cast< IEventListener* >( ( *itr ).get() )->VListen( hitEvent );
					dynamic_cast< IEventListener* >( ( *Eitr ).get() )->VListen( hitEvent );
				}
				++Eitr;
			}
			++itr;
		}
	}
	*/
	virtual void VListen( IEventData* eventData ){
		if( eventData->VGetType() == ET_BossDeath ){
		}
	}
	shared_ptr< PlayerActor > GetPlayer(){
		return mPlayer;
	}
};