#include "Collision.h"
#include "Math.h"
#include <math.h>
namespace FTLib{
	namespace Draw{
		namespace Math{
			namespace Collision2D{
				float vectorCross2D( Vector& first , Vector& second ){
					return first.x * second.y - first.y * second.x;
				}
				bool barToBar( Bar& bar1 , Bar& bar2 ){
					Draw::Vector v = Draw::Math::vectorSub( bar2.start , bar1.start );
					float cross = vectorCross2D( bar1.end , bar2.end );
					if( cross == 0.0f ){
						return false;
					}
					float cross1 = vectorCross2D( v , bar1.end );
					float cross2 = vectorCross2D( v , bar2.end );
					float t1 = cross2 / cross;
					float t2 = cross1 / cross;
					const float eps = 0.00001f;
					if( t1 + eps < 0 || t1 - eps > 1 || t2 + eps < 0 || t2 - eps > 1 ){
						return false;
					}else{
						return true;
					}
				}
				bool circleToCircle( Circle& circle1 , Circle& circle2 ){
					float xdis = circle1.pos.x - circle2.pos.x;
					float ydis = circle1.pos.y - circle2.pos.y;
					float distance = sqrt( xdis * xdis + ydis * ydis );
					return ( distance < circle1.r + circle2.r );

				}
				bool triangleToDot( Triangle& triangle , Vector& dotPos ){
					bool bool1 = ( vectorCross2D( vectorSub( dotPos , triangle.pos1 ) , vectorSub( triangle.pos2 , triangle.pos1 ) ) > 0 ); 
					bool bool2 = ( vectorCross2D( vectorSub( dotPos , triangle.pos2 ) , vectorSub( triangle.pos3 , triangle.pos2 ) ) > 0 ); 
					bool bool3 = ( vectorCross2D( vectorSub( dotPos , triangle.pos3 ) , vectorSub( triangle.pos1 , triangle.pos3 ) ) > 0 );
					if( ( bool1 && bool2 && bool3 ) || ( !bool1 && !bool2 && !bool3 ) ){
						return true;
					}else{
						return false;
					}
				}
				bool triangleToTriangle( Triangle& tri1 , Triangle& tri2 ){
					if( triangleToDot( tri1 , tri2.pos1 ) ) return true;
					if( triangleToDot( tri1 , tri2.pos2 ) ) return true;
					if( triangleToDot( tri1 , tri2.pos3 ) ) return true;
					if( triangleToDot( tri2 , tri1.pos1 ) ) return true;
					if( triangleToDot( tri2 , tri1.pos2 ) ) return true;
					if( triangleToDot( tri2 , tri1.pos3 ) ) return true;/*
					if( barToBar( Bar( tri1.pos1 , tri1.pos2 ) , Bar( tri2.pos1 , tri2.pos2 ) ) ) return true;
					if( barToBar( Bar( tri1.pos1 , tri1.pos2 ) , Bar( tri2.pos2 , tri2.pos3 ) ) ) return true;
					if( barToBar( Bar( tri1.pos1 , tri1.pos2 ) , Bar( tri2.pos3 , tri2.pos1 ) ) ) return true;
					if( barToBar( Bar( tri1.pos2 , tri1.pos3 ) , Bar( tri2.pos1 , tri2.pos2 ) ) ) return true;
					if( barToBar( Bar( tri1.pos2 , tri1.pos3 ) , Bar( tri2.pos2 , tri2.pos3 ) ) ) return true;
					if( barToBar( Bar( tri1.pos2 , tri1.pos3 ) , Bar( tri2.pos3 , tri2.pos1 ) ) ) return true;
					if( barToBar( Bar( tri1.pos3 , tri1.pos1 ) , Bar( tri2.pos1 , tri2.pos2 ) ) ) return true;
					if( barToBar( Bar( tri1.pos3 , tri1.pos1 ) , Bar( tri2.pos2 , tri2.pos3 ) ) ) return true;
					if( barToBar( Bar( tri1.pos3 , tri1.pos1 ) , Bar( tri2.pos3 , tri2.pos1 ) ) ) return true;*/
					return false;
				}
				bool boxToBox( Box& box1 , Box& box2 ){
					Triangle box1_1;
					box1_1.pos1 = box1.lefttop;
					box1_1.pos2 = box1.righttop;
					box1_1.pos3 = box1.leftbottom;
					Triangle box1_2;
					box1_2.pos1 = box1.righttop;
					box1_2.pos2 = box1.leftbottom;
					box1_2.pos3 = box1.rightbottom;
					Triangle box2_1;
					box2_1.pos1 = box2.lefttop;
					box2_1.pos2 = box2.righttop;
					box2_1.pos3 = box2.leftbottom;
					Triangle box2_2;
					box2_2.pos1 = box2.righttop;
					box2_2.pos2 = box2.leftbottom;
					box2_2.pos3 = box2.rightbottom;
					if( triangleToTriangle( box1_1 , box2_1 ) ) return true;
					if( triangleToTriangle( box1_1 , box2_2 ) ) return true;
					if( triangleToTriangle( box1_2 , box2_1 ) ) return true;
					if( triangleToTriangle( box1_2 , box2_2 ) ) return true;
					return false;
				}
				bool boxToBox_ZtoX( Box& box1 , Box& box2 ){
					box1.lefttop = Math::getVector( box1.lefttop.x , box1.lefttop.z , 0 );
					box1.leftbottom = Math::getVector( box1.leftbottom.x , box1.leftbottom.z , 0 );
					box1.righttop = Math::getVector( box1.righttop.x , box1.righttop.z , 0 );
					box1.rightbottom = Math::getVector( box1.rightbottom.x , box1.rightbottom.z , 0 );
					box2.lefttop = Math::getVector( box2.lefttop.x , box2.lefttop.z , 0 );
					box2.leftbottom = Math::getVector( box2.leftbottom.x , box2.leftbottom.z , 0 );
					box2.righttop = Math::getVector( box2.righttop.x , box2.righttop.z , 0 );
					box2.rightbottom = Math::getVector( box2.rightbottom.x , box2.rightbottom.z , 0 );
					return boxToBox( box1 , box2 );
				}
				bool boxToTriangle( Box& box , Triangle& triangle ){
					Triangle box_1;
					box_1.pos1 = box.lefttop;
					box_1.pos2 = box.righttop;
					box_1.pos3 = box.leftbottom;
					Triangle box_2;
					box_2.pos1 = box.righttop;
					box_2.pos2 = box.leftbottom;
					box_2.pos3 = box.rightbottom;
					if( triangleToTriangle( box_1 , triangle ) ) return true;
					if( triangleToTriangle( box_2 , triangle ) ) return true;
					return false;
				}
			}
		}
	}
}