#include "Font.h"

#include <string>
#include <vector>
#include <boost/algorithm/string.hpp>
namespace FTLib{
	namespace Draw{
		Font::Font(){
			mHandle = CreateFontToHandle(
				NULL,
				-1,
				-1 );
		}
		Font::Font( const char* fontname ){
			mHandle = CreateFontToHandle(
				fontname,
				-1,
				-1 );
		}
		Font::Font( const char* fontname , int size ){
			mHandle = CreateFontToHandle(
				fontname,
				size,
				-1 ,
				DX_FONTTYPE_ANTIALIASING );
		}
		Font::Font( const char* fontname , int size , Font::Type type ){
			mHandle = CreateFontToHandle(
				fontname,
				size,
				-1,
				type );
		}
		Font::~Font(){
			DeleteFontToHandle( mHandle );
		}
		void Font::draw(
			const char* text,
			int x,
			int y,
			int red,
			int green,
			int blue ){
				draw(
					text,
					x,
					y,
					red,
					green,
					blue,
					0,
					0,
					0 );
		}
		void Font::draw(
			const char* text,
			int x,
			int y,
			int red,
			int green,
			int blue,
			int edge_red,
			int edge_green,
			int edge_blue ){
				std::vector< std::string > v;
				std::string strText = text;
				boost::algorithm::split( v , strText , boost::algorithm::is_any_of( "\n" ) );
				int size;
				GetFontStateToHandle( NULL , &size , NULL , mHandle );
				for( int i = 0 ; i < v.size() ; ++i ){
					DrawStringToHandle(
						x,
						y + i * size,
						v.at( i ).c_str(),
						GetColor( red , green , blue ),
						mHandle,
						GetColor( edge_red , edge_green , edge_blue ) );
				}
		}
		void Font::centeringDraw(
			const char* text,
			int x,
			int y,
			int red,
			int green,
			int blue ){
				centeringDraw(
					text,
					x,
					y,
					red,
					green,
					blue,
					0,
					0,
					0 );
		}
		void Font::centeringDraw(
			const char* text,
			int x,
			int y,
			int red,
			int green,
			int blue,
			int edge_red,
			int edge_green,
			int edge_blue ){
				std::vector< std::string > v;
				std::string strText = text;
				boost::algorithm::split( v , strText , boost::algorithm::is_any_of( "\n" ) );
				int size;
				GetFontStateToHandle( NULL , &size , NULL , mHandle );
				for( int i = 0 ; i < v.size() ; ++i ){
					int width = GetDrawStringWidth( v[ i ].c_str() , v[ i ].size() );
					DrawStringToHandle(
						x - width / 2,
						y + i * size,
						v.at( i ).c_str(),
						GetColor( red , green , blue ),
						mHandle,
						GetColor( edge_red , edge_green , edge_blue ) );
				}
		}
		void Font::rectDraw(
			const char* text,
			int x,
			int y,
			int width,
			int height,
			int red,
			int green,
			int blue,
			int edge_red,
			int edge_green,
			int edge_blue )
		{
			std::string Text = text;
			std::string drawText;
			int strwidth = 0;
			int strlength = 0;
			int maxlength = Text.size();
			int line = 0;
			int size = GetFontSizeToHandle( mHandle );
			for( int i = 0 ; i < maxlength ; i += 2 ){
				std::string buf = drawText;
				buf += text[ i ];
				buf += text[ i + 1 ];
				int bufwidth = GetDrawStringWidth( buf.c_str() , buf.size() , 1 );
				if( bufwidth > width ){
					DrawString( x , y + line * size , drawText.c_str() , GetColor( red , green , blue ) , GetColor( edge_red , edge_green , edge_blue ) );
					drawText = "";
					buf = text[ i ];
					buf += text[ i + 1 ];
					++line;
				}
				drawText = buf;
				
			}
			DrawString( x , y + line * size , drawText.c_str() , GetColor( red , green , blue ) , GetColor( edge_red , edge_green , edge_blue ) );
		}
	}
}