#include "Pad.h"
namespace FTLib{
	namespace Input{
		Pad::Pad( int n ){
			mStates = new boost::circular_buffer< int >( 10 );
			mAxisStates = new boost::circular_buffer< Vector >( 10 );
			mRightAxisStates = new boost::circular_buffer< Vector >( 10);
			mNum = n;
		}
		Pad::~Pad(){
			delete mStates;
			delete mAxisStates;
			delete mRightAxisStates;
		}
		void Pad::updata(){
			mStates->push_front( GetJoypadInputState( mNum + 1 ) );
			mAxisStates->push_front( mUpdataAxis() );
			mRightAxisStates->push_front( mUpdataRightAxis() );
		}
		bool Pad::isOn( Button b ) const{
			if( mStates->size() == 0 ){
				return false;
			}else{
				if( mStates->at( 0 ) & b ){
					return true;
				}
			}
			return false;
		}
		bool Pad::isDown( Button b ) const{
			if( mStates->size() < 2 ){
				return false;
			}else{
				if( ! ( mStates->at( 1 ) & b ) ){
					if( mStates->at( 0 ) & b ){
						return true;
					}
				}
			}
			return false;
		}
		Vector Pad::joypadAxis() const{
			if( mAxisStates->size() == 0 ){
				return VGet( 0.0f , 0.0f , 0.0f );
			}else{
				return mAxisStates->at( 0 );
			}
		}
		Vector Pad::joypadAxisRight() const{
			if( mRightAxisStates->size() == 0 ){
				return VGet( 0.0f , 0.0f , 0.0f );
			}else{
				return mRightAxisStates->at( 0 );
			}
		}
		bool Pad::isDownAxis( Axis axis ) const{
			if( mAxisStates->size() < 2 ){
				return false;
			}else{
				switch( axis ){
					case AXIS_UP:
						if( -300 > mAxisStates->at( 0 ).y &&
							-300 < mAxisStates->at( 1 ).y ){
								return true;
						}else{
							return false;
						}
					case AXIS_DOWN:
						if( 300 < mAxisStates->at( 0 ).y &&
							300 > mAxisStates->at( 1 ).y ){
								return true;
						}else{
							return false;
						}
					case AXIS_LEFT:
						if( -300 > mAxisStates->at( 0 ).x &&
							-300 < mAxisStates->at( 1 ).x ){
								return true;
						}else{
							return false;
						}
					case AXIS_RIGHT:
						if( 300 < mAxisStates->at( 0 ).x &&
							300 > mAxisStates->at( 1 ).x ){
								return true;
						}else{
							return false;
						}
					default:
						return false;
				}
			}
		}
		bool Pad::isDownAxisRight( Axis axis ) const{
			if( mRightAxisStates->size() < 2 ){
				return false;
			}else{
				switch( axis ){
					case AXIS_UP:
						if( -300 > mRightAxisStates->at( 0 ).y &&
							-300 < mRightAxisStates->at( 1 ).y ){
								return true;
						}else{
							return false;
						}
					case AXIS_DOWN:
						if( 300 < mRightAxisStates->at( 0 ).y &&
							300 > mRightAxisStates->at( 1 ).y ){
								return true;
						}else{
							return false;
						}
					case AXIS_LEFT:
						if( -300 > mRightAxisStates->at( 0 ).x &&
							-300 < mRightAxisStates->at( 1 ).x ){
								return true;
						}else{
							return false;
						}
					case AXIS_RIGHT:
						if( 300 < mRightAxisStates->at( 0 ).x &&
							300 > mRightAxisStates->at( 1 ).x ){
								return true;
						}else{
							return false;
						}
					default:
						return false;
				}
			}
		}
		Vector Pad::mUpdataAxis(){
			int x , y;
			GetJoypadAnalogInput( &x , &y , mNum + 1 );
			return VGet( x , y , 0 );
		}
		Vector Pad::mUpdataRightAxis(){
			int x , y;
			GetJoypadAnalogInputRight( &x , &y , mNum + 1);
			return VGet( x , y , 0 );
		}
	}
}