#pragma once
#include "Std.h"
class Game;
class IActor;
enum ShotMovePatternType{
	SMPT_Unknown,
	SMPT_Standby,
	SMPT_StraightToP,
	SMPT_StraightToFirstPos,
	SMPT_AcceToP,
	SMPT_DeceToP,
	SMPT_Circle,
	SMPT_Random,
	SMPT_RandomAcce,
	SMPT_RandomDece,
	SMPT_Loop
};
class ShotPicts{
public:
	ShotPicts(){
		mPlayerShot = shared_ptr< Image >( new Image( "Pict/Game/PlayerShot.png" ) );
		mNormalShot = shared_ptr< Image >( new Image( "Pict/Game/Shot.png" ) );
		mPinkShot = shared_ptr< Image >( new Image( "Pict/Game/ShotPink.png" ) );
		mBlueShot = shared_ptr< Image >( new Image( "Pict/Game/ShotBlue.png" ) );
		mStarPinkShot = shared_ptr< Image >( new Image( "Pict/Game/StarPink.png" ) );
		mStarBlueShot = shared_ptr< Image >( new Image( "Pict/Game/StarBlue.png" ) );
		mMagicCircleRed = shared_ptr< Image >( new Image( "Pict/Game/RedMagicCircle.png" ) );
		mMagicCircleBlue = shared_ptr< Image >( new Image( "Pict/Game/BlueMagicCircle.png" ) );
		mPointItem = shared_ptr< Image >( new Image( "Pict/Game/PointItem.png" ) );
		mPowerItem = shared_ptr< Image >( new Image( "Pict/Game/PowerItem.png" ) );
		mLifeItem = shared_ptr< Image >( new Image( "Pict/Game/LifeItem.png" ) );
	}
	shared_ptr< Image > mPlayerShot;
	shared_ptr< Image > mNormalShot;
	shared_ptr< Image > mPinkShot;
	shared_ptr< Image > mBlueShot;
	shared_ptr< Image > mStarPinkShot;
	shared_ptr< Image > mStarBlueShot;
	shared_ptr< Image > mMagicCircleRed;
	shared_ptr< Image > mMagicCircleBlue;
	shared_ptr< Image > mPointItem;
	shared_ptr< Image > mPowerItem;
	shared_ptr< Image > mLifeItem;
};
extern ShotPicts* gShotPicts;
enum SoundType{
	SOUND_Bom,
	SOUND_Break,
	SOUND_Hit,
	SOUND_HitLow,
	SOUND_PointGet,
	SOUND_PowerGet,
	SOUND_None
};
class SoundDatas{
public:
	SoundDatas() : mBom( new Sound( "Sound/Game/Bom.wav" ) ),
	mBreak( new Sound( "Sound/Game/Break.wav" ) ),
	mHit( new Sound( "Sound/Game/Hit.wav" ) ),
	mHitLow( new Sound( "Sound/Game/Hit_Low.wav" ) ),
	mPointGet( new Sound( "Sound/Game/PointGet.wav" ) ),
	mPowerGet( new Sound( "Sound/Game/PowerGet.wav" ) ),
	mVolume( 128 ){
		mBreak->setVolume( mVolume );
	}
	shared_ptr< Sound > mBom;
	shared_ptr< Sound > mBreak;
	shared_ptr< Sound > mHit;
	shared_ptr< Sound > mHitLow;
	shared_ptr< Sound > mPointGet;
	shared_ptr< Sound > mPowerGet;
	int mVolume;
};
extern SoundDatas* gSoundDatas;
class IShotMovePattern{
public:
	virtual ~IShotMovePattern(){};
	virtual void OnUpdate( Game* game , IActor* enemy ) = 0;
	virtual ShotMovePatternType VGetType() const = 0;
	virtual void VSetType( ShotMovePatternType type ) = 0;
	virtual bool VIsEnd() const = 0;
	virtual void VInit() = 0;
};
class ShotMovePattern : public IShotMovePattern{
protected:
	ShotMovePatternType mType;
	bool mIsEndless;
	int mEndTime;
	int m_EndTime;
public:
	ShotMovePattern() : mType( SMPT_Unknown ) , mIsEndless( false ) , mEndTime( 0 ){};
	virtual void VSetType( ShotMovePatternType type ){ mType = type; }
	virtual bool VIsEnd() const{ return ( mEndTime <= 0 && mIsEndless == false ) ? true : false; }
	virtual ShotMovePatternType VGetType() const{ return mType; }
	virtual void VInit(){ mEndTime = m_EndTime; }
};
class SM_Standby : public ShotMovePattern{
public:
	SM_Standby( int time , bool endless = false ){
		VSetType( SMPT_Standby );
		mIsEndless = endless;
		mEndTime = time; 
		m_EndTime = time; 
	}
	virtual void OnUpdate( Game* game , IActor* enemy){ --mEndTime;	};
};
class SM_StraightToP : public ShotMovePattern{
private:
	int mSpeed;
	int mFreq;
	int m_Freq;
public:
	SM_StraightToP( int time , int speed , int freq , bool endless = false ){
		VSetType( SMPT_StraightToP );
		mIsEndless = endless;
		mEndTime = time;
		m_EndTime = time; 
		mSpeed = speed;
		mFreq = freq;
	}
	virtual void OnUpdate( Game* game , IActor* enemy);
};
class SM_StraightToFirstPos : public ShotMovePattern{
private:
	int mSpeed;
	int mFreq;
	int m_Freq;
	Math::Vector mPoint;
	bool mIsInit;
public:
	SM_StraightToFirstPos( int time , int speed , int freq , bool endless = false ){
		VSetType( SMPT_StraightToFirstPos );
		mIsEndless = endless;
		mEndTime = time; 
		m_EndTime = time; 
		mSpeed = speed;
		mFreq = freq;
	}
	virtual void OnUpdate( Game* game , IActor* enemy);
};
class SM_StrAcceToP : public ShotMovePattern{
private:
	int mSpeed;
	float mAcce;
	int mFreq;
	int m_Freq;
public:
	SM_StrAcceToP( int time , int speed , int acce , int freq , bool endless = false ){
		VSetType( SMPT_AcceToP );
		mIsEndless = endless;
		mEndTime = time;
		m_EndTime = time; 
		mSpeed = speed;
		mAcce = acce / 100.0f;
		mFreq = freq;
	}
	virtual void OnUpdate( Game* game , IActor* enemy);
};
class SM_StrDeceToP : public ShotMovePattern{
private:
	int mSpeed;
	float mDece;
	int mFreq;
	int m_Freq;
public:
	SM_StrDeceToP( int time , int speed , int dece , int freq , bool endless = false ){
		VSetType( SMPT_DeceToP );
		mIsEndless = endless;
		mEndTime = time;
		m_EndTime = time; 
		mSpeed = speed;
		mDece = dece / 100;
		mFreq = freq;
	}
	virtual void OnUpdate( Game* game , IActor* enemy);
};
class SM_Circle : public ShotMovePattern{
private:
	int mSpeed;
	int mNum;
	int mFreq;
	int m_Freq;
public:
	SM_Circle( int time , int speed , int num , int freq , bool endless = false ){
		VSetType( SMPT_Circle );
		mIsEndless = endless;
		mEndTime = time;
		m_EndTime = time; 
		mSpeed = speed;
		mNum = num;
		mFreq = freq;
	}
	virtual void OnUpdate( Game* game , IActor* enemy);
};
class SM_Random : public ShotMovePattern{
private:
	int mSpeed;
	int mNum;
	int mFreq;
	int m_Freq;
public:
	SM_Random( int time , int speed , int num , int freq , bool endless = false ){
		VSetType( SMPT_Random );
		mIsEndless = endless;
		mEndTime = time; 
		m_EndTime = time; 
		mSpeed = speed;
		mNum = num;
		mFreq = freq;
	}
	virtual void OnUpdate( Game* game , IActor* enemy);
};
class SM_RandomAcce : public ShotMovePattern{
private:
	int mSpeed;
	int mNum;
	int mFreq;
	int m_Freq;
	float mAcce;
public:
	SM_RandomAcce( int time , int speed , int num , int acce , int freq , bool endless = false ){
		VSetType( SMPT_RandomAcce );
		mIsEndless = endless;
		mEndTime = time;
		m_EndTime = time; 
		mSpeed = speed;
		mNum = num;
		mAcce = acce / 100;
		mFreq = freq;
	}
	virtual void OnUpdate( Game* game , IActor* enemy);
};
class SM_RandomDece : public ShotMovePattern{
private:
	int mSpeed;
	int mNum;
	int mFreq;
	int m_Freq;
	float mDece;
public:
	SM_RandomDece( int time , int speed , int num , int dece , int freq , bool endless = false ){
		VSetType( SMPT_RandomDece );
		mIsEndless = endless;
		mNum = num;
		mEndTime = time;
		m_EndTime = time; 
		mSpeed = speed;
		mDece = dece / 100;
		mFreq = freq;
	}
	virtual void OnUpdate( Game* game , IActor* enemy);
};
class SM_Loop : public ShotMovePattern{
public:
	SM_Loop(){
		VSetType( SMPT_Loop );
	}
	virtual void OnUpdate( Game* game , IActor* enemy){};
};
//GJe
//S{T[NeScS{@_T[NS
//
class SM_Ereka : public ShotMovePattern{
private:
	int mCnt;
	int mLineCnt;
	float mLineAngle[ 4 ];
	Math::Vector mLinePos[ 4 ];
	int mCircleCnt;
	enum ErekaMode{
		EREK_LineInit,
		EREK_Line,
		EREK_Unknown
	};
	ErekaMode mMode;
public:
	SM_Ereka(){
		mCnt = 0;
		mLineCnt = 0;
		mMode = EREK_LineInit;
		mIsEndless = true;
		mCircleCnt = 0;
	}
	virtual void OnUpdate( Game* game , IActor* enemy );
};