#pragma once
#define GAME_TITLE "ACX"
#define GAME_WINDOW_WIDTH 800
#define GAME_WINDOW_HEIGHT 600
#include "FTLib.h"
using namespace FTLib::Common;
using namespace FTLib::Algorithm;
using namespace FTLib::Draw;
using namespace FTLib::Input;
using namespace FTLib::Math;
using namespace FTLib::Music;
using namespace FTLib::Interface;

#include <boost/shared_ptr.hpp>
#include <boost/random.hpp>
#include <boost/lexical_cast.hpp>
using boost::shared_ptr;

#include <list>
#include <map>
#include <vector>

enum SequenceID{
	SEQ_None,
	SEQ_Init,
	SEQ_Opening,
	SEQ_MainMenu,
	SEQ_Loading,
	SEQ_Game,
	SEQ_Quit
};
class IrisBaseGame;
class CProcessManager;
class IActor;
class ISequence{
public:
	virtual void OnUpdate( IrisBaseGame* mGame , unsigned int elpsedFrame ) = 0;
	virtual void OnDraw(){};
};
typedef unsigned int ActorId;
typedef std::map< ActorId , shared_ptr< IActor > > ActorMap;
enum EventType{
	ET_Unknown,
	ET_Hit,
	ET_SequenceMove,
	ET_BossDeath
};
class IEventData{
public:
	virtual ~IEventData(){}
	virtual EventType VGetType() = 0;
};
class BaseEventData : public IEventData{
protected:
	EventType mType;
public:
	virtual ~BaseEventData(){}
	virtual EventType VGetType(){ return mType; }
};
class EVE_Hit : public BaseEventData{
private:
	IActor* mObj1;
	IActor* mObj2;
public:
	EVE_Hit( IActor* obj1 , IActor* obj2 ) : mObj1( obj1 ) , mObj2( obj2 ){ mType = ET_Hit; }
	IActor* Obj1() const{ return mObj1; }
	IActor* Obj2() const{ return mObj2; }
};
class IEventListener{
public:
	~IEventListener(){}
	virtual void VListen( IEventData* eventData ) = 0;
};
typedef std::vector< shared_ptr< IEventListener > > Listeners;
class Actor;
class IrisBaseGame{
protected:
	CProcessManager* mProcessManager;
	//CRandom mRandom;
	ActorMap mActorMap;
	ActorId mLastActorID;
	shared_ptr< ISequence > mSequence;
	SequenceID mNext;
	bool mIsPause;
	FourTree::CLiner4TreeManager< IActor > mCollTree;
	typedef std::vector< FourTree::OBJECT_FOR_TREE < IActor >* > Objects;
	Objects mObjects;
public:
	IrisBaseGame();
	virtual ~IrisBaseGame(){
		Objects::iterator itr = mObjects.begin();
		while( itr != mObjects.end() ){
			( *itr )->Remove();
			SAFE_DELETE( ( *itr ) );
			++itr;
		}
	}
	void Pause( bool pause ){ mIsPause = pause; }

	bool OnUpdate( unsigned int elapsedFrame );
	
	void SequenceMove( SequenceID next ){ mNext = next; }
	ISequence* GetSequence() const{ return mSequence.get(); }
};
class GameApp{
public:
	GameApp(){
		srand( 123 );
		mGame = shared_ptr< IrisBaseGame >( new IrisBaseGame() );
	}
	~GameApp(){}
	std::string const& GetString( const int id ) const;
	bool OnUpdate( unsigned int deltaMilliseconds ){
		return mGame->OnUpdate( deltaMilliseconds );
	}
	shared_ptr< IrisBaseGame > GetGame(){ return mGame; };
	int GetRandom( int min , int max ) const{
		return rand() % ( max - min + 1 ) + min;
	}
private:
	std::map< int , std::string > mStrings;
	shared_ptr< IrisBaseGame > mGame;
};
extern GameApp* gApp;